import * as elements from '@yellicode/elements';
import * as opts from './options';
import { CodeWriter, TextWriter } from '@yellicode/templating';
import { NamespaceDefinition, ClassDefinition, AccessModifier, InterfaceDefinition, EnumDefinition, EnumMemberDefinition, MethodDefinition, ParameterDefinition, PropertyDefinition, StructDefinition } from './model';
export declare class CSharpWriter extends CodeWriter {
    private typeNameProvider;
    private definitionBuilder;
    private commentWriter;
    /**
     * Constructor. Creates a new CSharpWriter instance using the TextWriter and options provided.
     * @param writer The template's current TextWriter.
     * @param options Optional: the global options for this writer.
     */
    constructor(writer: TextWriter, options?: opts.WriterOptions);
    /**
     * Writes 1 or more using directives, each on a new line.
     * @param values A collection of strings, typically namespace names.
     */
    writeUsingDirectives(...values: string[]): void;
    /**
     * Writes an indented block of code, wrapped in opening and closing brackets.
     * @param contents A callback function that writes the contents.
     */
    writeCodeBlock(contents: (writer: CSharpWriter) => void): void;
    /**
     * Writes an indented block of code, wrapped in a namespace declaration and opening and closing brackets.
     * @param definition The namespace definition. Not that an XML doc summary is not supported.
     * @param contents A callback function that writes the namespace contents.
     */
    writeNamespaceBlock(definition: NamespaceDefinition, contents: (writer: CSharpWriter) => void): void;
    /**
     * Writes an indented block of code, wrapped in a namespace declaration and opening and closing brackets.
     * @param pack A package or model that represents the namespace.
     * @param contents A callback function that writes the namespace contents.
     * @param options An optional NamespaceOptions object.
     */
    writeNamespaceBlock(pack: elements.Package, contents: (writer: CSharpWriter) => void, options?: opts.NamespaceOptions): void;
    /**
     * Writes a block of code, wrapped in a class declaration and opening and closing brackets.
     * This function does not write class members.
     * @param definition The class definition.
     * @param contents A callback function that writes the class contents.
     */
    writeClassBlock(definition: ClassDefinition, contents: (writer: CSharpWriter) => void): void;
    /**
   * Writes a block of code, wrapped in a class declaration and opening and closing brackets.
   * This function does not write class members.
   * @param cls The class.
   * @param contents A callback function that writes the class contents.
   * @param options An optional ClassOptions object.
   */
    writeClassBlock(cls: elements.Class, contents: (writer: CSharpWriter) => void, options?: opts.ClassOptions): void;
    /**
    * Writes a block of code, wrapped in a struct declaration and opening and closing brackets.
    * This function does not write struct members.
    * @param definition The struct definition.
    * @param contents A callback function that writes the struct contents.
    */
    writeStructBlock(definition: StructDefinition, contents: (writer: CSharpWriter) => void): void;
    /**
   * Writes a block of code, wrapped in a struct declaration and opening and closing brackets.
   * This function does not write struct members.
   * @param cls The struct type.
   * @param contents A callback function that writes the struct contents.
   * @param options An optional StructOptions object.
   */
    writeStructBlock(cls: elements.Type, contents: (writer: CSharpWriter) => void, options?: opts.StructOptions): void;
    /**
     * Writes a block of code, wrapped in an interface declaration and opening and closing brackets.
     * This function does not write interface members.
     * @param definition The interface definition.
     * @param contents  A callback function that writes the interface contents.
     */
    writeInterfaceBlock(definition: InterfaceDefinition, contents: (writer: CSharpWriter) => void): void;
    /**
     * Writes a block of code, wrapped in an interface declaration and opening and closing brackets.
     * This function does not write interface members.
     * @param iface The interface.
     * @param contents A callback function that writes the interface contents.
     * @param options An optional InterfaceOptions object.
     */
    writeInterfaceBlock(iface: elements.Interface, contents: (writer: CSharpWriter) => void, options?: opts.InterfaceOptions): void;
    /**
     * Writes a block of code, wrapped in an enum declaration and opening and closing brackets.
     * This function does not write enumeration members. Use the writeEnumMember function
     * to write each individual member or the writeEnumeration function to write the full enumeration.
     * @param definition The enumeration definition.
     * @param contents A callback function that writes the enumeration contents.
     */
    writeEnumerationBlock(definition: EnumDefinition, contents: (writer: CSharpWriter) => void): void;
    /**
    * Writes a block of code, wrapped in an enum declaration and opening and closing brackets.
    * This function does not write enumeration members. Use the writeEnumMember function
    * to write each individual member or the writeEnumeration function to write the full enumeration.
    * @param element The enumeration.
    * @param contents A callback function that writes the enumeration contents.
    * @param options An optional EnumerationOptions object.
    */
    writeEnumerationBlock(enumeration: elements.Enumeration, contents: (writer: CSharpWriter) => void, options?: opts.EnumOptions): void;
    /**
     *  Writes a full enumeration, including members.
     * @param definition The enumeration definition.
     */
    writeEnumeration(definition: EnumDefinition): void;
    /**
     * Writes a full enumeration, including members.
     * @param element The enumeration.
     * @param options An optional EnumOptions object.
     */
    writeEnumeration(enumeration: elements.Enumeration, options?: opts.EnumOptions): void;
    /**
     * Writes an individual enumeration member.
     * @param definition The enumeration member definition.
     */
    writeEnumMember(definition: EnumMemberDefinition): void;
    /**
     * Writes an individual enumeration member.
     * @param literal The EnumerationLiteral for which to write the member.
     * @param options An optional EnumMemberOptions object.
     * @param isLast Set to true if this is the last member of the enumeration to be written (avoiding
     * a trailing comma).
     */
    writeEnumMember(literal: elements.EnumerationLiteral, options?: opts.EnumMemberOptions, isLast?: boolean): void;
    /**
     * Writes a method declaration without a body. Use this function to generate interface methods.
     * @param definition The method definition.
     */
    writeMethodDeclaration(definition: MethodDefinition): void;
    /**
   * Writes a method declaration without a body. Use this function to generate interface methods.
   * @param operation The operation for which to write the method.
   * @param options An optional MethodOptions object.
   */
    writeMethodDeclaration(operation: elements.Operation, options?: opts.MethodOptions): void;
    /**
    * Writes an indented block of code, wrapped in a method declaration and opening and closing brackets.
    * @param method The operation for which to write the method.
    * @param contents A callback function that writes the operation contents. This callback will not be invoked
    * if the method is abstract.
     */
    writeMethodBlock(method: MethodDefinition, contents: (writer: CSharpWriter) => void): void;
    /**
    * Writes an indented block of code, wrapped in a method declaration and opening and closing brackets.
    * @param operation The operation for which to write the method.
    * @param contents A callback function that writes the operation contents. This callback will not be invoked
    * if the method is abstract.
    * @param options An optional MethodOptions object.
    */
    writeMethodBlock(operation: elements.Operation, contents: (writer: CSharpWriter) => void, options?: opts.MethodOptions): void;
    /**
     * Writes the input and output parameters of a method.
     * @param params The parameter definitions.
     */
    writeInOutParameters(params: ParameterDefinition[]): void;
    /**
   * Writes the input and output parameters (all parameters except the return parameter) of a method.
   * @param params A collection of parameters.
   * @param options An optional MethodOptions object.
   */
    writeInOutParameters(params: elements.Parameter[], options?: opts.MethodOptions): void;
    /**
     * Writes an auto property with a getter and a setter.
     * @param property The property definition.
     */
    writeAutoProperty(property: PropertyDefinition): void;
    /**
    * Writes an auto property with a getter and - if the property is not ReadOnly or Derived - a setter.
    * This function can be used for both Class- and and Interface properties.
    */
    writeAutoProperty(property: elements.Property, options?: opts.PropertyOptions): void;
    /**
     *  Writes a property code block using optional callback functions for writing the getter and setter contents.
     * @param property The property definition.
     * @param getterContents An optional callback function that writes the getter code.
     * @param setterContents An optional callback function that writes the setter code.
     */
    writePropertyBlock(property: PropertyDefinition, getterContents: () => void | null, setterContents: () => void | null): void;
    /**
    *  Writes a property code block using optional callback functions for writing the getter and setter contents.
    * @param property The property.
    * @param getterContents An optional callback function that writes the getter code.
    * @param setterContents An optional callback function that writes the setter code.
    * @param options An optional PropertyOptions object.
    */
    writePropertyBlock(property: elements.Property, getterContents: () => void | null, setterContents: () => void | null, options?: opts.PropertyOptions): void;
    private writePropertyStart(definition);
    /**
    * Writes the type's access modifier to the output with a trailing whitespace.
    * @param definition The type definition.
    */
    writeAccessModifier(definition: {
        accessModifier?: AccessModifier;
    }): void;
    /**
    * Writes the visibility to the output with a trailing whitespace. If the visibilty is null or
    * not supported by C#, nothing will be written.
    * @param visibilityKind A VisibilityKind value. This value can be null.
    */
    writeAccessModifier(visibilityKind: elements.VisibilityKind | null): void;
    private writeInherits(inherits?);
    private writeImplements(hasGeneralizations, impl?);
    /**
     * Gets the name of the type. This function uses the current typeNameProvider for resolving
     * the type name.
     * @param type Any element that derives from Type.
     */
    getTypeName(type: elements.Type | null): string | null;
    /**
    * Gets the type name of the typed element. This function uses the current typeNameProvider for resolving
    * the type name.
    * @param typedElement Any element that has a type, such as a Property or Parameter.
    */
    getTypeName(typedElement: elements.TypedElement | null): string | null;
    /**
     * Writes a method declaration without a body.
     * @deprecated Use the writeMethodDeclaration() function instead.
     */
    writeInterfaceMethod(operation: elements.Operation, options?: opts.MethodOptions): void;
    /**
     * Writes an indented block of code, wrapped in a method declaration and opening and closing brackets.
     * @deprecated Use the writeMethodBlock() function instead.
     */
    writeClassMethodBlock(operation: elements.Operation, contents: (writer: CSharpWriter) => void, options?: opts.MethodOptions): void;
    writeXmlDocSummary(comments: string[]): void;
    writeXmlDocSummary(text: string): void;
    writeXmlDocParagraph(text: string): void;
    writeXmlDocLines(lines: string[]): void;
    writeDelimitedCommentParagraph(text: string): void;
    writeDelimitedCommentLines(lines: string[]): void;
    private joinWrite<TItem>(collection, separator, getStringFunc);
}
